//
//  WeChatShareTool.swift
//  HmmProject
//
//  Created by ZCC on 2019/4/10.
//  Copyright © 2019 Hmm. All rights reserved.
//

import UIKit

/// 分享模型
class ShareModel: NSObject {
    
    /// 标题
    var title: String?
    /// 分享链接
    var url: String?
    /// 缩略图数据
    var thumbData: Data?
    /// 描述
    var description: String?
    
    /// 图片分享 设置这个后其余参数无效
    var fileData: Data?
}

// MARK: -微信分享类型
enum WechatShareType: Int {
    /// 0-好友列表 1-朋友圈
    case session = 0, timeline
}

// MARK: -微信分享工具类
class WeChatShareTool: NSObject {
    
    /// 是否已安装微信
    static var isWeChatInstall: Bool {
        guard let url = URL.init(string: "weixin://") else { return false }
        return UIApplication.shared.canOpenURL(url)
    }
    
    /// 注册微信分享
    ///
    /// - Parameter appID: 微信AppId
    class func registerWechat(appID: String) {
        
        WeChatShareTool._appID = appID
    }
    
    /// 微信分享
    ///
    /// - Parameters:
    ///   - type: 类型
    ///   - shareModel: 分享模型
    ///   - block: 回调
    class func shareToWeChat(type: WechatShareType, shareModel: ShareModel, block: WeChatBlock?) {
        
        guard let appKey = WeChatShareTool._appID, WeChatShareTool.isWeChatInstall else {
            WeChatShareTool._appID = nil
            DispatchQueue.main.async {
                block?(-1)
            }
            return
        }
        var weChatMessageInfo: [String: Any] = [
            "result": "1",
            "returnFromApp": "0",
            "scene": "\(type.rawValue)",
            "sdkver": "1.5",
            "command": "1010"
        ]
        if let fileData = shareModel.fileData {
            
            // 图片 详情图
            weChatMessageInfo["objectType"] = "2"
            weChatMessageInfo["fileData"] = fileData
            
        } else {
            
            // 标题
            if let title = shareModel.title {
                weChatMessageInfo["title"] = title
            }
            // 描述
            if let description = shareModel.description {
                weChatMessageInfo["description"] = description
            }
            // 缩略图 小于32k的Data
            if let thumbData = shareModel.thumbData {
                weChatMessageInfo["thumbData"] = thumbData
            }
            // 跳转链接
            if let shareUrl = shareModel.url {
                // 分享类型 5-图文 2-图
                weChatMessageInfo["objectType"] = "5"
                weChatMessageInfo["mediaUrl"] = shareUrl
            }
        }
        if weChatMessageInfo["objectType"] == nil {
            DispatchQueue.main.async {
                block?(-1)
            }
            return
        }
        WeChatShareTool._block = block
        var weChatMessage: [String: Any] = [appKey: weChatMessageInfo]
        if let oldText = UIPasteboard.general.k_oldText {
            weChatMessage["old_text"] = oldText
        }
        guard let data = try? PropertyListSerialization.data(fromPropertyList: weChatMessage, format: .binary, options: .init()), let url = URL(string: "weixin://app/\(appKey)/sendreq/?") else {
            DispatchQueue.main.async {
                block?(-1)
            }
            return
        }
        UIPasteboard.general.setData(data, forPasteboardType: "content")
        UIApplication.shared.openURL(url)
    }
    
    /// 微信分享回调
    ///
    /// - Parameter url: url
    /// - Returns: 是否成功
    class func handleOpenURL(_ url: URL) -> Bool {
        guard let urlScheme = url.scheme, let appId = WeChatShareTool._appID else { return false }
        if let data = UIPasteboard.general.data(forPasteboardType: "content"), let dict = try? PropertyListSerialization.propertyList(from: data, format: nil) as? [String: Any], urlScheme.hasPrefix("wx") {
            
            guard let info = dict[appId] as? [String: Any], let result = info["result"] as? String, let resultCode = Int(result) else {
                DispatchQueue.main.async {
                    WeChatShareTool._block?(-1)
                }
                return false
            }
            DispatchQueue.main.async {
                WeChatShareTool._block?(resultCode)
            }
            do {
                WeChatShareTool._block = nil
                WeChatShareTool._appID = nil
            }
        }
        return false
    }
    
    /// 0-成功 -1失败 -2用户取消
    typealias WeChatBlock = (_ result: Int)->Void
    /// 回调
    static private var _block: WeChatBlock?
    /// 微信ID
    static private var _appID: String!
}

extension URL {
    
    var k_queryDictionary: [String: String] {
        guard let components = URLComponents(url: self, resolvingAgainstBaseURL: false), let queryItems = components.queryItems else {
            return [:]
        }
        let items = queryItems.compactMap { $0.value != nil ? ($0.name, $0.value!) : nil }
        return Dictionary(items, uniquingKeysWith: { _, last in last })
    }
}

extension UIPasteboard {
    /// Fetch old text on pasteboard
    var k_oldText: String? {
        /// From iOS 8 to iOS 11, UIPasteboardTypeListString contains two elements: public.text, public.utf8-plain-text
        guard let typeListString = UIPasteboard.typeListString as? [String] else { return nil }
        guard UIPasteboard.general.contains(pasteboardTypes: typeListString) else { return nil }
        return UIPasteboard.general.string
    }
}
